﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace Fuel.Core
{
    public class SignatureValidator
    {
        private readonly string _accessKeySecret;
        private readonly ILogger<SignatureValidator> _logger;

        public SignatureValidator(string accessKeySecret, ILogger<SignatureValidator> logger)
        {
            _accessKeySecret = accessKeySecret;
            _logger = logger;
        }

        /// <summary>
        /// 验证传入请求的签名是否有效。
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public bool ValidateSignature(HttpRequest request)
        {
            try
            {
                var signMethod = request.Headers["sign_method"].ToString();
                var secretId = request.Headers["secret_id"].ToString();
                var nonce = request.Headers["nonce"].ToString();
                var timestamp = request.Headers["timestamp"].ToString();
                var signature = request.Headers["signature"].ToString();

                if (string.IsNullOrEmpty(signMethod) || string.IsNullOrEmpty(secretId) ||
                    string.IsNullOrEmpty(nonce) || string.IsNullOrEmpty(timestamp) ||
                    string.IsNullOrEmpty(signature))
                {
                    _logger.LogError("缺少签名信息");
                    return false;
                }

                // 查询字符串
                var queryString = HttpUtility.ParseQueryString(request.QueryString.ToString());
                var queryParameters = queryString.AllKeys
                    .OrderBy(k => k)
                    .Select(k => $"{k}={queryString[k]}")
                    .Aggregate((current, next) => current + "&" + next);

                // 待签字符串
                var stringToSign = $"sign_method={signMethod}&secret_id={secretId}&nonce={nonce}&timestamp={timestamp}&{queryParameters}";

                // 根据 RFC3986 对字符串进行 URL 编码
                var encodedString = Uri.EscapeDataString(stringToSign);

                // 计算 HMAC-SM3 哈希（此处用 HMACSHA256 作为占位符）
                var calculatedSignature = HmacSm3(_accessKeySecret, encodedString);
                var base64Signature = Convert.ToBase64String(calculatedSignature);
                return base64Signature.Equals(signature, StringComparison.Ordinal);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "签名验证过程中发生错误。");
                return false;
            }
        }

        /// <summary>
        /// 使用 HMAC-SM3 算法计算哈希值（此处用 HMACSHA256 作为占位符）
        /// </summary>
        /// <param name="key"></param>
        /// <param name="message"></param>
        /// <returns></returns>
        private byte[] HmacSm3(string key, string message)
        {
            using (var hmacsha = new HMACSHA256(Encoding.UTF8.GetBytes(key)))
            {
                return hmacsha.ComputeHash(Encoding.UTF8.GetBytes(message));
            }
        }
    }
}
